'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

exports.raw = undefined;
exports.default = loader;

const fs = require('fs-extra');
const path = require('path');
const glob = require('glob');
const Memoryfs = require('memory-fs');

const appDirectory = fs.realpathSync(process.cwd());

const isApp = process.env.jsbridgeBuildType == 'app';
const isWap =
  process.env.jsbridgeBuildType == 'wap' ||
  process.env.jsbridgeBuildType == 'web';

const picshareModuleMap = {};

let picshareModuleId = 0;

const picshareDistRootPath = 'picshare';

function loader(content) {
  const self = this;
  const callback = this.async();
  const picshareName = getpicshareNameFromManifest(this);
  const filePath = this.resource;
  let out = '';
  let promise;

  if (!(picshareName in picshareModuleMap)) {
    picshareModuleMap[picshareName] = picshareModuleId;
    // picshareModuleId++;
  } else if (process.env.NODE_ENV == 'production') {
    throw new Error('[picshare-loader] 命名重复，已有模块命名为' + picshareName);
  }

  if (isWap) {
    out += `module.exports = {
        openScreenShot:function(){}
      }`;
    promise = Promise.resolve();
  }

  if (isApp) {
    const rootDir = appDirectory;
    promise = BundlepicshareAssets(filePath, picshareModuleId++, picshareName)
      .then(({ picshareModuleId, picshareName }) => {
        const staticBuildFilePath = path.resolve(
          rootDir,
          `build/picshare/${picshareModuleId}/index/static/`
        );

        const htmlContent = require('./htmlTemplate');

        const destpicshareRootPath = 'picshare';

        const destpicshareValidDir = path.join(destpicshareRootPath, picshareName);

        const htmlDir = path.join(destpicshareValidDir, 'index.html');

        self.emitFile(htmlDir, htmlContent);

        const staticFileArr = glob.sync(
          path.join(staticBuildFilePath, '**/*.*')
        );

        const fileRelaPathArr = staticFileArr.map(filePath => {
          return filePath.split(staticBuildFilePath)[1];
        });

        const readFilePromiseArr = staticFileArr.map(filePath => {
          return new Promise((resolve, reject) => {
            fs.readFile(filePath).then(value => {
              resolve(value);
            });
          });
        });

        const fileEmitPms = Promise.all(readFilePromiseArr).then(
          (files, index) => {
            files.forEach((content, index) => {
              const destPath = path.join(
                destpicshareValidDir,
                'static',
                fileRelaPathArr[index]
              );

              this.emitFile(destPath, content);
            });
          }
        );

        const modalPath = `picshare/${picshareName}/index.html`;

        out = `
        var appSNC = require("@mfelibs/universal-framework").default;
        require("@mfelibs/universal-framework/src/libs/apis/URLNavigateTo");
          module.exports = {
            openScreenShot: function(param) {
              console.log("__picShare loader log__")
              var fParam = {}
              fParam=Object.assign(fParam, {
                path: "picshare/${picshareName}/index.html"
              }, param);
              appSNC.URLNavigateTo({
                actionType:27,
                ext:fParam
              });
            }
          }`;

        return fileEmitPms.then(() => {
          return {
            rootDir,
            picshareModuleId
          };
        });
      })
      .then(removeDir);
  }

  if (promise) {
    promise.then(() => {
      callback(null, out);
    });
  } else {
    return '';
  }
}

function BundlepicshareAssets(filePath, picshareModuleId, picshareName) {
  picshareModuleId = picshareModuleId.toString();

  const webpack = require('webpack');
  const getWebpackConfig = require('./webpackConfig.js');
  const tempSrcPath = path.resolve(
    appDirectory,
    path.join('build/picshare/', picshareModuleId)
  );
  const tempSrcFilePath = path.resolve(tempSrcPath, 'index.js');
  let webpackConfig = getWebpackConfig({
    entry: 'index'
  });

  if (!fs.existsSync(tempSrcPath)) {
    fs.mkdirpSync(tempSrcPath);
    // fs.mkdirSync(tempSrcFilePath);
  }

  const code = `
  var  _a  = require('${filePath}') ;
    import appSNC from '@mfelibs/universal-framework';
    import '@mfelibs/universal-framework/src/libs/apis/closeWindow';
    import '@mfelibs/universal-framework/src/libs/apis/onRendered';

    function closeModal() {
      appSNC.closeWindow()
    }

    appSNC.ready(function(data) {
      _a.picShare(data, {closeModal: closeModal}, "root").show()
      appSNC.onRendered();
    })

    appSNC.onRendered();
  `;

  // 创建临时文件
  fs.writeFileSync(tempSrcFilePath, code);

  // webpackConfig.entry = tempSrcFilePath;

  const webpackEntry = [];
  webpackEntry.push(
    path.resolve('./node_modules/webpack-marauder/webpack/polyfills.js')
  );
  webpackEntry.push(tempSrcFilePath);

  webpackConfig.entry.index = webpackEntry;

  webpackConfig.output.path = path.resolve(
    appDirectory,
    path.join('build/picshare/', picshareModuleId),
    'index'
  );

  const compiler = webpack(webpackConfig);
  // compiler.outputFileSystem = new Memoryfs();

  return new Promise((resolve, reject) => {
    compiler.run((err, stats) => {
      if (err) return reject(err);
      resolve({
        picshareModuleId,
        picshareName
      });
    });
  });
}

function getpicshareNameFromManifest(loaderContext) {
  var manifestPath = path.join(loaderContext.context, 'manifest.json');
  let loaderName = '';

  if (fs.existsSync(manifestPath) != true) {
    throw new Error(
      '[picshare-loader] 没有找到picshare对应的manifest.json，在picshare入口文件同目录建立manifest.json文件'
    );
  }

  try {
    const manifest = require(manifestPath);
    loaderName = manifest.name; //打包的时候输出目录用
  } catch (ex) {
    throw new Error('[picshare-loader] picshare对应manifest识别失败！');
  }

  if (loaderName === '') {
    throw new Error('[picshare-loader] manifest name字段不得为空！');
  }

  return loaderName;
}

function removeDir({ rootDir, picshareModuleId }) {
  const tempDir = path.resolve(rootDir, path.join('build/picshare/', picshareModuleId));

  // const distDir = path.resolve(rootDir, "dist" + picshareModuleId);

  if (fs.existsSync(tempDir)) {
    return Promise.all([fs.emptyDir(tempDir)]).then(() => {
      fs.rmdirSync(tempDir);
    });
  }
}

var raw = (exports.raw = false);
